import java.io.*;


/**
 * ChillMapper is the main class for an application that displays
 * wind chill information on a map of Virginia.
 *
 * @author  Prof. David Bernstein, James Madison University
 * @version 1.0
 */
public class ChillMapper
{
    private static Map         map;
    

    /**
     * The entry point of the application.
     *
     * If there are no command-line arguments, this program will
     * process the data in the file today.txt.
     *
     * If there is one command-line argument, it must contain the
     * name of the file to process.
     *
     * If the number of command-line arguments is a multiple of 3 then
     * they contain the weather data to process.
     *
     * @param args   The command-line arguments
     */
    public static void main(String[] args)
    {
       map = new Map("va");       

       if      (args.length == 0)       processFile("today.txt");
       else if (args.length == 1)       processFile(args[0]);
       else if ((args.length % 3) == 0) processCommandLineArguments(args);
    }

    /**
     * Add a calculated wind chill value to the Map (if both the
     * temperature and the wind speed are valid double values)
     *
     * @param station     The station code
     * @param temperature A String representation of the temperature
     * @param windSpeed   A String representation of the wind speed
     */
    private static void addToMap(String station, 
                                 String temperature, String windSpeed)
    {
       double    t, v;
       
       try
       {
          t = Double.parseDouble(temperature);
          v = Double.parseDouble(windSpeed);
          map.setTemperature(station, Weather.windChillNA(t, v));       
       }
       catch (NumberFormatException nfe)
       {
          // Ignore
       }
    }

    /**
     * Process data contained in the command-line arguments
     *
     * @param args   The command-line arguments
     */
    private static void processCommandLineArguments(String[] args)
    {
       for (int i=0; i<args.length; i+=3)
       {
          addToMap(args[i], args[i+1], args[i+2]);             
       }
    }

    /**
     * Process data contained in a file
     *
     * @param file   The name of the file containing the data
     */
    private static void processFile(String file)
    {
       BufferedReader       in;       
       String    line;
       String[]  tokens;

       try
       {
          in    = new BufferedReader(new FileReader(new File(file)));
          while ((line=in.readLine()) != null)
          {
             tokens = line.split(" ");             
             addToMap(tokens[0], tokens[1], tokens[2]);             
          }
       }
       catch (IOException ioe)
       {
          // Ignore
       }
    }
}
