import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;
import javax.swing.*;


/**
 * A component that displays information about temperatures on 
 * a map
 *
 * @author  Prof. David Bernstein, James Madison University
 * @version 1.0
 */
public class MapComponent extends JComponent
{
    // Attributes
    private ArrayList<Shape>                shapes; 
    private BufferedReader                  in;       
    private HashMap<String, Point2D.Double> stations;
    private HashMap<String, Double>         temperatures;
    private JFrame                          frame;
    private JLabel                          maxLabel, minLabel;    
    private JTextArea                       textArea;
    
    // Constants 
    private final Font FIXED_WIDTH = new Font(Font.MONOSPACED, Font.PLAIN, 12);

    /**
     * Explicit Value Constructor
     *
     * @param region   The name of the region (e.g., "va")
     */
    public MapComponent(String region)    
    {
       super();
       setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 16));
       

       // Initialization
       shapes       = new ArrayList<Shape>();       
       stations     = new HashMap<String, Point2D.Double>();
       temperatures = new HashMap<String, Double>();

       // Input
       try
       {
          readMap(region);       
          readStations(region);
       }
       catch (Exception e)
       {
          e.printStackTrace();
          
       }
    }
    
    
    /**
     * Render this component
     *
     * @param g   The rendering engine to use
     */
    public void paint(Graphics g)
    {
       double           value;       
       FontMetrics      metrics;
       Graphics2D       g2;
       Iterator<String> keys;
       Point2D          lonlat;       
       String           key, text;
			
       
              
       super.paint(g);
       g2 = (Graphics2D)g;
       // Use antialiasing for shapes
       g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                           RenderingHints.VALUE_ANTIALIAS_ON);
       // Use high-quality rendering
       g2.setRenderingHint(RenderingHints.KEY_RENDERING,
                           RenderingHints.VALUE_RENDER_QUALITY);
       // Use low-level "antialiasing for LCDs" for text
       g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                           RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
       metrics = g2.getFontMetrics(getFont());    
		 
		 
       // Render the map
       for(Shape s: shapes)
       {
          g2.setColor(Color.GRAY);
          g2.fill(s);
          g2.setColor(Color.BLACK);
          g2.draw(s);    
       }

       // Render the temperatures
       g2.setFont(getFont());
       keys = temperatures.keySet().iterator();
       while (keys.hasNext())
       {
          key    = keys.next();
          value  = temperatures.get(key);
          lonlat = stations.get(key);
          if (lonlat != null)
          {
             g2.setColor(Color.BLUE);
             text = String.format("%6.1f",value);
             g2.drawString(text,
                    (float)lonlat.getX()-(float)(metrics.stringWidth(text)/2.),
                    (float)lonlat.getY()-(float)(metrics.getHeight()/2.));
          }
       }
    }


    /**
     * Read the map
     *
     * @param region   The name of the region
     */
    private void readMap(String region) throws Exception
    {
       Shape                 shape;       
       String                id, line, token, type;
       StringTokenizer       st;
       
       in = new BufferedReader(new FileReader(region+".txt"));       
       
       try 
       {
          while ((line = in.readLine()) != null)
          {
             st = new StringTokenizer(line, "\t ");
             
             token = st.nextToken();
             type  = st.nextToken();

             // Read the ID if there is one
             id = null;
             
             if (st.hasMoreTokens())  token = st.nextToken();
             if (st.hasMoreTokens())  id = st.nextToken();

             
             shape = readPolygon();
             shapes.add(shape);                
          }
			 
          in.close();
       }
       catch (IOException ioe)
       {
          // Try again
       }
    }
    

    

    /**
     * Read a Polygon
     */
    private Shape readPolygon() throws IOException
    {
       boolean               keepReading;
       double                latitude, longitude;
       GeneralPath           polygon;       
       String                line, token;
       StringTokenizer       st;
       
       polygon = null;
       keepReading = true;

       while (keepReading)
       {
          line = in.readLine();
          
          if ((line == null) || line.startsWith("END"))
          {
             keepReading = false;
          }
          else
          {
             st = new StringTokenizer(line, "\t ");

             token = st.nextToken();
             longitude = Double.parseDouble(token);
             
             token = st.nextToken();
             latitude = Double.parseDouble(token);

             if (polygon == null) 
             {
                polygon = new GeneralPath();
                polygon.moveTo(longitude, latitude);                
             }
             else polygon.lineTo(longitude, latitude);
          }
       }

       return polygon;       
    }




    /**
     * Read the stations
     *
     * @param region   The name of the region
     */
    private void readStations(String region) throws Exception
    {
       BufferedReader        in;
       double                lat, lon, sign;
       double[]              p;
       String                icao4, line, token;
       StringTokenizer       st;
       
       in = new BufferedReader(new FileReader("stations-"+region+".txt"));       
       
       try 
       {
          while ((line = in.readLine()) != null)
          {
             st = new StringTokenizer(line, "\t");
             icao4  = st.nextToken();
             lon = Double.parseDouble(st.nextToken());
             lat = Double.parseDouble(st.nextToken());
             stations.put(icao4, new Point2D.Double(lon, lat));
          }
       }
       catch (Exception e)
       {
       }
    }
    

    /**
     * Set the temperature for a particular station
     *
     * @param icao4        The 4-letter ICAO code for the station
     * @param temperature  The temperature
     */
    public void setTemperature(String icao4, double temperature)
    {
       if (icao4 == null) icao4 = "null";

       temperatures.put(icao4, temperature);
       repaint();
    }

}
